"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

require("core-js/modules/es.array.iterator");

require("core-js/modules/es.array.sort");

require("core-js/modules/es.promise");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.UiMain = void 0;

function _defineProperty2() {
  const data = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

  _defineProperty2 = function () {
    return data;
  };

  return data;
}

function _cache() {
  const data = require("@teambit/cache");

  _cache = function () {
    return data;
  };

  return data;
}

function _cli() {
  const data = require("@teambit/cli");

  _cli = function () {
    return data;
  };

  return data;
}

function _component() {
  const data = require("@teambit/component");

  _component = function () {
    return data;
  };

  return data;
}

function _express() {
  const data = require("@teambit/express");

  _express = function () {
    return data;
  };

  return data;
}

function _graphql() {
  const data = require("@teambit/graphql");

  _graphql = function () {
    return data;
  };

  return data;
}

function _chalk() {
  const data = _interopRequireDefault(require("chalk"));

  _chalk = function () {
    return data;
  };

  return data;
}

function _harmony() {
  const data = require("@teambit/harmony");

  _harmony = function () {
    return data;
  };

  return data;
}

function _logger() {
  const data = require("@teambit/logger");

  _logger = function () {
    return data;
  };

  return data;
}

function _pubsub() {
  const data = _interopRequireDefault(require("@teambit/pubsub"));

  _pubsub = function () {
    return data;
  };

  return data;
}

function _utils() {
  const data = require("bit-bin/dist/utils");

  _utils = function () {
    return data;
  };

  return data;
}

function _fsExtra() {
  const data = _interopRequireDefault(require("fs-extra"));

  _fsExtra = function () {
    return data;
  };

  return data;
}

function _getPort() {
  const data = _interopRequireDefault(require("get-port"));

  _getPort = function () {
    return data;
  };

  return data;
}

function _path() {
  const data = require("path");

  _path = function () {
    return data;
  };

  return data;
}

function _util() {
  const data = require("util");

  _util = function () {
    return data;
  };

  return data;
}

function _webpack() {
  const data = _interopRequireDefault(require("webpack"));

  _webpack = function () {
    return data;
  };

  return data;
}

function _events() {
  const data = require("./events");

  _events = function () {
    return data;
  };

  return data;
}

function _createRoot() {
  const data = require("./create-root");

  _createRoot = function () {
    return data;
  };

  return data;
}

function _exceptions() {
  const data = require("./exceptions");

  _exceptions = function () {
    return data;
  };

  return data;
}

function _start() {
  const data = require("./start.cmd");

  _start = function () {
    return data;
  };

  return data;
}

function _uiBuild() {
  const data = require("./ui-build.cmd");

  _uiBuild = function () {
    return data;
  };

  return data;
}

function _uiServer() {
  const data = require("./ui-server");

  _uiServer = function () {
    return data;
  };

  return data;
}

function _ui() {
  const data = require("./ui.aspect");

  _ui = function () {
    return data;
  };

  return data;
}

function _openBrowser() {
  const data = require("./open-browser");

  _openBrowser = function () {
    return data;
  };

  return data;
}

function _webpack2() {
  const data = _interopRequireDefault(require("./webpack/webpack.config"));

  _webpack2 = function () {
    return data;
  };

  return data;
}

class UiMain {
  constructor(pubsub, config, graphql, uiRootSlot, express, onStartSlot, componentExtension, cache, logger) {
    this.pubsub = pubsub;
    this.config = config;
    this.graphql = graphql;
    this.uiRootSlot = uiRootSlot;
    this.express = express;
    this.onStartSlot = onStartSlot;
    this.componentExtension = componentExtension;
    this.cache = cache;
    this.logger = logger;
    (0, _defineProperty2().default)(this, "createUiServerStartedEvent", (targetHost, targetPort, uiRoot) => {
      return new (_events().UiServerStartedEvent)(Date.now(), targetHost, targetPort, uiRoot);
    });
  }
  /**
   * create a build of the given UI root.
   */


  async build(uiRootName) {
    const [name, uiRoot] = this.getUi(uiRootName); // TODO: @uri refactor all dev server related code to use the bundler extension instead.

    const config = (0, _webpack2().default)(uiRoot.path, [await this.generateRoot(await uiRoot.resolveAspects(_ui().UIRuntime.name), name)], uiRoot.name);
    const compiler = (0, _webpack().default)(config);
    const compilerRun = (0, _util().promisify)(compiler.run.bind(compiler));
    return compilerRun();
  }
  /**
   * create a Bit UI runtime.
   */


  async createRuntime({
    uiRootName,
    pattern,
    dev,
    port,
    rebuild
  }) {
    const [name, uiRoot] = this.getUi(uiRootName);
    this.componentExtension.setHostPriority(name);

    const uiServer = _uiServer().UIServer.create({
      express: this.express,
      graphql: this.graphql,
      uiRoot,
      uiRootExtension: name,
      ui: this,
      logger: this.logger
    });

    const targetPort = await this.getPort(port);

    if (dev) {
      await uiServer.dev({
        port: targetPort
      });
    } else {
      await this.buildIfChanged(name, uiRoot, rebuild);
      await this.buildIfNoBundle(name, uiRoot);
      await uiServer.start({
        port: targetPort
      });
    }

    this.pubsub.pub(_ui().UIAspect.id, this.createUiServerStartedEvent(this.config.host, targetPort, uiRoot));
    if (uiRoot.postStart) await uiRoot.postStart({
      pattern
    });
    await this.invokeOnStart();
    return uiServer;
  }

  async getPort(port) {
    if (port) return port;
    return this.config.port || this.selectPort();
  }
  /**
   * Events
   */


  /**
   * bind to ui server start event.
   */
  registerOnStart(onStartFn) {
    this.onStartSlot.register(onStartFn);
    return this;
  }

  async invokeOnStart() {
    const promises = this.onStartSlot.values().map(fn => fn());
    await Promise.all(promises);
  }
  /**
   * register a UI slot.
   */


  registerUiRoot(uiRoot) {
    return this.uiRootSlot.register(uiRoot);
  }
  /**
   * get a UI runtime instance.
   */


  getUi(uiRootName) {
    if (uiRootName) {
      const root = this.getUiRootOrThrow(uiRootName);
      return [uiRootName, root];
    }

    const uis = this.uiRootSlot.toArray();
    if (uis.length === 1) return uis[0];
    const uiRoot = uis.find(([, root]) => root.priority);
    if (!uiRoot) throw new (_exceptions().UnknownUI)('default', this.uiRootSlot.toArray().map(([id]) => id));
    return uiRoot;
  }

  getUiRootOrThrow(uiRootName) {
    const uiSlot = this.uiRootSlot.get(uiRootName);
    if (!uiSlot) throw new (_exceptions().UnknownUI)(uiRootName, this.uiRootSlot.toArray().map(([id]) => id));
    return uiSlot;
  }

  createLink(aspectDefs, rootExtensionName) {
    return (0, _createRoot().createRoot)(aspectDefs, rootExtensionName);
  }
  /**
   * generate the root file of the UI runtime.
   */


  async generateRoot(aspectDefs, rootExtensionName, runtimeName = _ui().UIRuntime.name, rootAspect = _ui().UIAspect.id) {
    const contents = await (0, _createRoot().createRoot)(aspectDefs, rootExtensionName, rootAspect, runtimeName);
    const filepath = (0, _path().resolve)((0, _path().join)(__dirname, `${runtimeName}.root${(0, _utils().sha1)(contents)}.js`));
    if (_fsExtra().default.existsSync(filepath)) return filepath;

    _fsExtra().default.outputFileSync(filepath, contents);

    return filepath;
  }

  async selectPort() {
    const [from, to] = this.config.portRange;
    return (0, _getPort().default)({
      port: _getPort().default.makeRange(from, to)
    });
  }

  async buildUiHash(uiRoot, runtime = 'ui') {
    const aspects = await uiRoot.resolveAspects(runtime);
    aspects.sort((a, b) => a.aspectPath > b.aspectPath ? 1 : -1);
    const hash = aspects.map(aspect => {
      return [aspect.aspectPath, aspect.runtimePath].join('');
    });
    return (0, _utils().sha1)(hash.join(''));
  }

  async buildIfChanged(name, uiRoot, force) {
    const hash = await this.buildUiHash(uiRoot);
    const hashed = await this.cache.get(uiRoot.path);
    if (hash === hashed && !force) return;
    if (hash !== hashed) this.logger.console(`${uiRoot.configFile} has been changed. Rebuilding UI assets for '${_chalk().default.cyan(uiRoot.name)}'`);
    this.logger.console(`Building UI assets for '${_chalk().default.cyan(uiRoot.name)}'`);
    await this.build(name);
    await this.cache.set(uiRoot.path, hash);
  }

  async buildIfNoBundle(name, uiRoot) {
    const config = (0, _webpack2().default)(uiRoot.path, [await this.generateRoot(await uiRoot.resolveAspects(_ui().UIRuntime.name), name)], uiRoot.name);
    if (_fsExtra().default.pathExistsSync(config.output.path)) return;
    if (_fsExtra().default.readdirSync(config.output.path).length > 0) return;
    const hash = await this.buildUiHash(uiRoot);
    await this.build(name);
    await this.cache.set(uiRoot.path, hash);
  }

  async openBrowser(url) {
    const openBrowser = new (_openBrowser().OpenBrowser)(this.logger);
    openBrowser.open(url);
  }

  static async provider([pubsub, cli, graphql, express, componentExtension, cache, loggerMain], config, [uiRootSlot, onStartSlot]) {
    // aspectExtension.registerRuntime(new RuntimeDefinition('ui', []))
    const logger = loggerMain.createLogger(_ui().UIAspect.id);
    const ui = new UiMain(pubsub, config, graphql, uiRootSlot, express, onStartSlot, componentExtension, cache, logger);
    cli.register(new (_start().StartCmd)(ui, logger, pubsub));
    cli.register(new (_uiBuild().UIBuildCmd)(ui));
    return ui;
  }

}

exports.UiMain = UiMain;
(0, _defineProperty2().default)(UiMain, "defaultConfig", {
  portRange: [3000, 3100],
  host: 'localhost'
});
(0, _defineProperty2().default)(UiMain, "runtime", _cli().MainRuntime);
(0, _defineProperty2().default)(UiMain, "dependencies", [_pubsub().default, _cli().CLIAspect, _graphql().GraphqlAspect, _express().ExpressAspect, _component().ComponentAspect, _cache().CacheAspect, _logger().LoggerAspect]);
(0, _defineProperty2().default)(UiMain, "slots", [_harmony().Slot.withType(), _harmony().Slot.withType()]);

_ui().UIAspect.addRuntime(UiMain);

//# sourceMappingURL=ui.main.runtime.js.map